@@ -58,6 +58,8 @@ extern "C" {
 
 R_LIB_VERSION_HEADER(r_cons);
 
+#define R_CONS_CMD_DEPTH 100
+
 typedef int (*RConsGetSize)(int *rows);
 typedef int (*RConsGetCursor)(int *rows);
 typedef bool (*RConsIsBreaked)(void);
@@ -254,6 +256,10 @@ typedef struct r_cons_palette_t {
 	RColor graph_trufae;
 	RColor graph_traced;
 	RColor graph_current;
+        RColor graph_diff_match;
+        RColor graph_diff_unmatch;
+        RColor graph_diff_unknown;
+        RColor graph_diff_new;
 } RConsPalette;
 
 typedef struct r_cons_printable_palette_t {
@@ -321,6 +327,10 @@ typedef struct r_cons_printable_palette_t {
 	char *graph_box2;
 	char *graph_box3;
 	char *graph_box4;
+	char *graph_diff_match;
+	char *graph_diff_unmatch;
+	char *graph_diff_unknown;
+	char *graph_diff_new;
 	char *graph_true;
 	char *graph_false;
 	char *graph_trufae;
@@ -398,8 +408,9 @@ typedef struct r_cons_canvas_t {
 #define RUNE_CURVE_CORNER_TR "╮"
 #define RUNE_CURVE_CORNER_BR "╯"
 #define RUNE_CURVE_CORNER_BL "╰"
-
+#define RUNE_LONG_LINE_HORIZ "―"
 #define UTF_CIRCLE "\u25EF"
+#define UTF_BLOCK "\u2588"
 
 typedef char *(*RConsEditorCallback)(void *core, const char *file, const char *str);
 typedef int (*RConsClickCallback)(void *core, int x, int y);
@@ -421,6 +432,7 @@ typedef struct r_cons_context_t {
 	RStack *break_stack;
 	RConsEvent event_interrupt;
 	void *event_interrupt_data;
+	int cmd_depth;
 
 	// Used for per-task logging redirection
 	RLogCallback log_callback; // TODO: RList of callbacks
@@ -482,6 +494,7 @@ typedef struct r_cons_t {
 	char *pager;
 	int blankline;
 	char *highlight;
+	bool enable_highlight;
 	int null; // if set, does not show anything
 	int mouse;
 	int is_wine;
@@ -715,6 +728,11 @@ enum {
 	LINE_NOSYM_HORIZ
 };
 
+typedef enum {
+	INSERT_MODE = 'i',
+	CONTROL_MODE = 'c'
+} RViMode;
+
 #define DOT_STYLE_NORMAL 0
 #define DOT_STYLE_CONDITIONAL 1
 #define DOT_STYLE_BACKEDGE 2
@@ -939,6 +957,7 @@ R_API int r_cons_get_column(void);
 R_API char *r_cons_message(const char *msg);
 R_API void r_cons_set_title(const char *str);
 R_API bool r_cons_enable_mouse(const bool enable);
+R_API void r_cons_enable_highlight(const bool enable);
 R_API void r_cons_bind(RConsBind *bind);
 R_API const char* r_cons_get_rune(const ut8 ch);
 #endif
@@ -995,6 +1014,7 @@ typedef int (*RLineCompletionCb)(RLineCompletion *completion, RLineBuffer *buf,
 struct r_line_comp_t {
 	bool opt;
 	size_t args_limit;
+	bool quit;
 	RPVector args; /* <char *> */
 	RLineCompletionCb run;
 	void *run_user;
@@ -1026,7 +1046,9 @@ struct r_line_t {
 	int (*hist_down)(void *user);
 	char *contents;
 	bool zerosep;
-	bool vi_mode;
+	bool enable_vi_mode;
+	int vi_mode;
+	bool prompt_mode;
 	RLinePromptType prompt_type;
 	int offset_hist_index;
 	int file_hist_index;
@@ -1135,7 +1157,7 @@ typedef struct r_panels_t {
 	bool autoUpdate;
 	bool mouse_on_edge_x;
 	bool mouse_on_edge_y;
-	RPanelsMenu *panelsMenu;
+	RPanelsMenu *panels_menu;
 	Sdb *db;
 	Sdb *rotate_db;
 	Sdb *almighty_db;
